using Microsoft.AspNetCore.Mvc;
using Swashbuckle.AspNetCore.Annotations;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.ViewModels;
using cuqmbr.TravelGuide.Domain.Enums;
using cuqmbr.TravelGuide.Application.Addresses;
using cuqmbr.TravelGuide.Application.Addresses.Commands.AddAddress;
using cuqmbr.TravelGuide.Application.Addresses.Queries.GetAddressesPage;
using cuqmbr.TravelGuide.Application.Addresses.Queries.GetAddress;
using cuqmbr.TravelGuide.Application.Addresses.Commands.UpdateAddress;
using cuqmbr.TravelGuide.Application.Addresses.Commands.DeleteAddress;
using cuqmbr.TravelGuide.Application.Addresses.ViewModels;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("addresses")]
public class AddressesController : ControllerBase
{
    [HttpPost]
    [SwaggerOperation("Add an address")]
    [SwaggerResponse(
        StatusCodes.Status201Created, "Object successfuly created",
        typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<ActionResult<AddressDto>> Add(
        [FromBody] AddAddressViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return StatusCode(
            StatusCodes.Status201Created,
            await Mediator.Send(
                new AddAddressCommand()
                {
                    Name = viewModel.Name,
                    Longitude = viewModel.Longitude,
                    Latitude = viewModel.Latitude,
                    VehicleType = VehicleType.FromName(viewModel.VehicleType),
                    CityGuid = viewModel.CityUuid
                },
                cancellationToken));
    }

    [HttpGet]
    [SwaggerOperation("Get a list of all addresses")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful",
        typeof(PaginatedList<AddressDto>))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<PaginatedList<AddressDto>> GetPage(
        [FromQuery] PageQuery pageQuery, [FromQuery] SearchQuery searchQuery,
        [FromQuery] SortQuery sortQuery, 
        [FromQuery] GetAddressesPageFilterViewModel filterQuery,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new GetAddressesPageQuery()
            {
                PageNumber = pageQuery.PageNumber,
                PageSize = pageQuery.PageSize,
                Search = searchQuery.Search,
                Sort = sortQuery.Sort,
                LongitudeGreaterOrEqualThan = 
                    filterQuery.LongitudeGreaterOrEqualThan,
                LongitudeLessOrEqualThan = 
                    filterQuery.LongitudeLessOrEqualThan,
                LatitudeGreaterOrEqualThan = 
                    filterQuery.LatitudeGreaterOrEqualThan,
                LatitudeLessOrEqualThan = 
                    filterQuery.LatitudeLessOrEqualThan,
                VehicleType = VehicleType.FromName(filterQuery.VehicleType),
                CountryGuid = filterQuery.CountryUuid,
                RegionGuid = filterQuery.RegionUuid,
                CityGuid = filterQuery.CityUuid
            },
            cancellationToken);
    }

    [HttpGet("{uuid:guid}")]
    [SwaggerOperation("Get an address by uuid")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<AddressDto> Get(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(new GetAddressQuery() { Guid = uuid },
            cancellationToken);
    }

    [HttpPut("{uuid:guid}")]
    [SwaggerOperation("Update an address")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<AddressDto> Update(
        [FromRoute] Guid uuid,
        [FromBody] UpdateAddressViewModel viewModel,
        CancellationToken cancellationToken)
    {
            return await Mediator.Send(
                new UpdateAddressCommand()
                {
                    Guid = uuid,
                    Name = viewModel.Name,
                    Longitude = viewModel.Longitude,
                    Latitude = viewModel.Latitude,
                    VehicleType = VehicleType.FromName(viewModel.VehicleType),
                    CityGuid = viewModel.CityUuid
                },
                cancellationToken);
    }

    [HttpDelete("{uuid:guid}")]
    [SwaggerOperation("Delete an address")]
    [SwaggerResponse(StatusCodes.Status204NoContent, "Request successful")]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(AddressDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<IActionResult> Delete(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        await Mediator.Send(
            new DeleteAddressCommand() { Guid = uuid },
            cancellationToken);
        return StatusCode(StatusCodes.Status204NoContent);
    }
}
