using Microsoft.AspNetCore.Mvc;
using Swashbuckle.AspNetCore.Annotations;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.ViewModels;
using cuqmbr.TravelGuide.Application.Cities;
using cuqmbr.TravelGuide.Application.Cities.Commands.AddCity;
using cuqmbr.TravelGuide.Application.Cities.Queries.GetCitiesPage;
using cuqmbr.TravelGuide.Application.Cities.Queries.GetCity;
using cuqmbr.TravelGuide.Application.Cities.Commands.UpdateCity;
using cuqmbr.TravelGuide.Application.Cities.Commands.DeleteCity;
using cuqmbr.TravelGuide.Application.Cities.ViewModels;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("cities")]
public class CitiesController : ControllerBase
{
    [HttpPost]
    [SwaggerOperation("Add a city")]
    [SwaggerResponse(
        StatusCodes.Status201Created, "Object successfuly created",
        typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<ActionResult<CityDto>> Add(
        [FromBody] AddCityViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return StatusCode(
            StatusCodes.Status201Created,
            await Mediator.Send(
                new AddCityCommand()
                {
                    Name = viewModel.Name,
                    RegionGuid = viewModel.RegionGuid
                },
                cancellationToken));
    }

    [HttpGet]
    [SwaggerOperation("Get a list of all cities")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful",
        typeof(PaginatedList<CityDto>))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<PaginatedList<CityDto>> GetPage(
        [FromQuery] PageQuery pageQuery, [FromQuery] SearchQuery searchQuery,
        [FromQuery] SortQuery sortQuery, 
        [FromQuery] GetCitiesPageFilterViewModel filterQuery,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new GetCitiesPageQuery()
            {
                PageNumber = pageQuery.PageNumber,
                PageSize = pageQuery.PageSize,
                Search = searchQuery.Search,
                Sort = sortQuery.Sort,
                CountryGuid = filterQuery.CountryUuid,
                RegionGuid = filterQuery.RegionUuid
            },
            cancellationToken);
    }

    [HttpGet("{uuid:guid}")]
    [SwaggerOperation("Get a city by uuid")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<CityDto> Get(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(new GetCityQuery() { Guid = uuid },
            cancellationToken);
    }

    [HttpPut("{uuid:guid}")]
    [SwaggerOperation("Update a city")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<CityDto> Update(
        [FromRoute] Guid uuid,
        [FromBody] UpdateCityViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new UpdateCityCommand()
            {
                Guid = uuid,
                Name= viewModel.Name,
                RegionGuid = viewModel.RegionUuid
            },
            cancellationToken);
    }

    [HttpDelete("{uuid:guid}")]
    [SwaggerOperation("Delete a city")]
    [SwaggerResponse(StatusCodes.Status204NoContent, "Request successful")]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(CityDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<IActionResult> Delete(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        await Mediator.Send(
            new DeleteCityCommand() { Guid = uuid },
            cancellationToken);
        return StatusCode(StatusCodes.Status204NoContent);
    }
}
