using Microsoft.AspNetCore.Mvc;
using Swashbuckle.AspNetCore.Annotations;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.ViewModels;
using cuqmbr.TravelGuide.Application.Regions.ViewModels;
using cuqmbr.TravelGuide.Application.Regions;
using cuqmbr.TravelGuide.Application.Regions.Commands.AddRegion;
using cuqmbr.TravelGuide.Application.Regions.Queries.GetRegionsPage;
using cuqmbr.TravelGuide.Application.Regions.Queries.GetRegion;
using cuqmbr.TravelGuide.Application.Regions.Commands.UpdateRegion;
using cuqmbr.TravelGuide.Application.Regions.Commands.DeleteRegion;
using cuqmbr.TravelGuide.Application.Regions.ViewModels;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("regions")]
public class RegionsController : ControllerBase
{
    [HttpPost]
    [SwaggerOperation("Add a region")]
    [SwaggerResponse(
        StatusCodes.Status201Created, "Object successfuly created",
        typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<ActionResult<RegionDto>> Add(
        [FromBody] AddRegionViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return StatusCode(
            StatusCodes.Status201Created,
            await Mediator.Send(
                new AddRegionCommand()
                {
                    Name = viewModel.Name
                },
                cancellationToken));
    }

    [HttpGet]
    [SwaggerOperation("Get a list of all regions")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful",
        typeof(PaginatedList<RegionDto>))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<PaginatedList<RegionDto>> GetPage(
        [FromQuery] PageQuery pageQuery, [FromQuery] SearchQuery searchQuery,
        [FromQuery] SortQuery sortQuery, 
        [FromQuery] GetRegionsPageFilterViewModel filterQuery,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new GetRegionsPageQuery()
            {
                PageNumber = pageQuery.PageNumber,
                PageSize = pageQuery.PageSize,
                Search = searchQuery.Search,
                Sort = sortQuery.Sort,
                CountryGuid = filterQuery.CountryUuid
            },
            cancellationToken);
    }

    [HttpGet("{uuid:guid}")]
    [SwaggerOperation("Get a region by uuid")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<RegionDto> Get(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(new GetRegionQuery() { Guid = uuid },
            cancellationToken);
    }

    [HttpPut("{uuid:guid}")]
    [SwaggerOperation("Update a region")]
    [SwaggerResponse(
        StatusCodes.Status200OK, "Request successful", typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Object already exists",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Parent object not found",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<RegionDto> Update(
        [FromRoute] Guid uuid,
        [FromBody] UpdateRegionViewModel viewModel,
        CancellationToken cancellationToken)
    {
        return await Mediator.Send(
            new UpdateRegionCommand()
            {
                Guid = uuid,
                Name = viewModel.Name,
                CountryGuid = viewModel.CountryUuid
            },
            cancellationToken);
    }

    [HttpDelete("{uuid:guid}")]
    [SwaggerOperation("Delete a region")]
    [SwaggerResponse(StatusCodes.Status204NoContent, "Request successful")]
    [SwaggerResponse(
        StatusCodes.Status400BadRequest, "Input data validation error",
        typeof(HttpValidationProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status401Unauthorized, "Unauthorized to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status403Forbidden,
        "Not enough privileges to perform an action",
        typeof(ProblemDetails))]
    [SwaggerResponse(
        StatusCodes.Status404NotFound, "Object not found", typeof(RegionDto))]
    [SwaggerResponse(
        StatusCodes.Status500InternalServerError, "Internal server error",
        typeof(ProblemDetails))]
    public async Task<IActionResult> Delete(
        [FromRoute] Guid uuid,
        CancellationToken cancellationToken)
    {
        await Mediator.Send(
            new DeleteRegionCommand() { Guid = uuid },
            cancellationToken);
        return StatusCode(StatusCodes.Status204NoContent);
    }
}
